<?php

namespace Drupal\acquia_perz;

use Acquia\Hmac\KeyInterface;
use Drupal\acquia_connector\Subscription;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\ContentEntityType;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\State\StateInterface;
use Drupal\field\Entity\FieldConfig;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;

/**
 * Entity helper service.
 */
class EntityHelper {

  const ENTITY_CONFIG_NAME = 'acquia_perz.entity_config';

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The state storage service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Representation of the current HTTP request.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The HTTP client.
   *
   * @var \Drupal\acquia_perz\ClientFactory
   */
  protected $perzClient;

  /**
   * The File Url Generator.
   *
   * @var \Drupal\core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * Acquia Subscription Data.
   *
   * @var array
   */
  protected $subscription = [];

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   Entity field manager service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection to be used.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\acquia_perz\ClientFactory $perz_client
   *   A Guzzle client object.
   * @param \Drupal\core\File\FileUrlGeneratorInterface $file_url_generator
   *   The File url generator service.
   * @param \Drupal\acquia_connector\Subscription $subscription
   *   Acquia Subscription.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    EntityFieldManagerInterface $entity_field_manager,
    RendererInterface $renderer,
    DateFormatterInterface $date_formatter,
    TimeInterface $time,
    ConfigFactoryInterface $config_factory,
    Connection $database,
    StateInterface $state,
    RequestStack $request_stack,
    ModuleHandlerInterface $module_handler,
    ClientFactory $perz_client,
    FileUrlGeneratorInterface $file_url_generator,
    Subscription $subscription
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->renderer = $renderer;
    $this->dateFormatter = $date_formatter;
    $this->time = $time;
    $this->configFactory = $config_factory;
    $this->database = $database;
    $this->state = $state;
    $this->requestStack = $request_stack;
    $this->moduleHandler = $module_handler;
    $this->perzClient = $perz_client;
    $this->fileUrlGenerator = $file_url_generator;
    $subscription_data = $subscription->getSubscription();
    if (isset($subscription_data['acquia_perz'])) {
      $this->subscription = $subscription_data['acquia_perz'];
    }
  }

  /**
   * Is valid entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current entity.
   */
  public function isValidEntity(EntityInterface $entity) {
    $view_modes = $this->getEntityViewModesSettingValue($entity);
    if (empty($view_modes)) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Returns the value of the entity view modes setting.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current entity.
   *
   * @return array
   *   The setting value.
   */
  public function getEntityViewModesSettingValue(EntityInterface $entity): array {
    $view_modes = $this->configFactory
      ->get('acquia_perz.entity_config')->get("view_modes.{$entity->getEntityTypeId()}.{$entity->bundle()}");
    if ($view_modes) {
      return array_keys($view_modes);
    }
    return [];
  }

  /**
   * Get list of entity uuids.
   *
   * Format:
   * [
   *   [
   *     'entity_type_id' => '...',
   *     'entity_uuid' => '...',
   *   ]
   * ].
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param array $bundles
   *   List of bundles of entity type.
   * @param int $offset
   *   The query offset.
   * @param int $limit
   *   The query limit.
   *
   * @return array
   *   Returns a list of entities by its entity type id.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntitiesUuids($entity_type_id, array $bundles, $offset, $limit) {
    $entity_ids = $this->getEntitiesIds($entity_type_id, $bundles, $offset, $limit);
    $entity_type = $this->entityTypeManager
      ->getStorage($entity_type_id)
      ->getEntityType();
    $entity_items = [];
    foreach ($entity_ids as $entity_id) {
      $entity_uuid = $this->getEntityUuidById($entity_type, $entity_id);
      $entity_items[] = [
        'entity_type_id' => $entity_type_id,
        'entity_uuid' => $entity_uuid,
      ];
    }
    return $entity_items;
  }

  /**
   * Get entity query.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param array $bundles
   *   List of bundles of entity type.
   * @param int $offset
   *   The query offset.
   * @param int $limit
   *   The query limit.
   *
   * @return array
   *   Returns entity query.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntitiesQuery($entity_type_id, array $bundles, int $offset = NULL, int $limit = NULL) {
    // Check only bundles with at least one view mode activated
    // besides 'acquia_perz_preview_image' view mode.
    $available_bundles = [];
    foreach ($bundles as $bundle => $view_modes) {
      $view_modes = array_keys($view_modes);
      if (count($view_modes) === 1
        && in_array('acquia_perz_preview_image', $view_modes)) {
        continue;
      }
      $available_bundles[] = $bundle;
    }
    // Skip entity type without activated bundles.
    if (empty($available_bundles)) {
      return [];
    }
    $bundle_property_name = $this
      ->entityTypeManager
      ->getStorage($entity_type_id)
      ->getEntityType()
      ->getKey('bundle');
    $query = $this
      ->entityTypeManager
      ->getStorage($entity_type_id)
      ->getQuery();
    // For single-bundle entity types like 'user'
    // we don't use bundle related property.
    if (!empty($bundle_property_name)) {
      $query = $query->condition($bundle_property_name, $available_bundles, 'IN');
    }
    if ($entity_type_id === 'user') {
      $query = $query->condition('uid', 0, '<>');
    }
    if ($offset !== NULL && $limit !== NULL) {
      $query->range($offset, $limit);
    }

    return $query;
  }

  /**
   * Get entity ids by entity type id and passed bundles.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param array $bundles
   *   List of bundles of entity type.
   * @param int $offset
   *   The query offset.
   * @param int $limit
   *   The query limit.
   *
   * @return array
   *   Returns entity ids.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntitiesIds($entity_type_id, array $bundles, int $offset = NULL, int $limit = NULL) {
    $query = $this->getEntitiesQuery($entity_type_id, $bundles, $offset, $limit);
    return array_values($query->execute());
  }

  /**
   * Get entities query count by entity type and available bundles.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param array $bundles
   *   List of bundles and its view modes of entity type.
   *   Format:
   *   [
   *    'page' => [
   *       'default' => 1,
   *       'view_mode2' => 1,
   *       ...
   *    ]
   *   ].
   * @param int|null $offset
   *   The query offset.
   * @param int|null $limit
   *   The query limit.
   *
   * @return int
   *   Returns number of entities.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getCountByEntityTypeId($entity_type_id, array $bundles, int $offset = NULL, int $limit = NULL) {
    $query = $this->getEntitiesQuery($entity_type_id, $bundles, $offset, $limit);
    return (int) $query->count()->execute();
  }

  /**
   * Get number of all entities by its entity types.
   *
   * @param array $entity_types
   *   List of entity types.
   *
   * @return int
   *   Returns number of all available entities.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getTotalCount(array $entity_types) {
    $total_count = 0;
    foreach ($entity_types as $entity_type_id => $bundles) {
      $total_count += $this->getCountByEntityTypeId($entity_type_id, $bundles);
    }
    return $total_count;
  }

  /**
   * Get single entity uuid by entity id and its entity type.
   *
   * @param \Drupal\Core\Entity\Annotation\ContentEntityType $entity_type
   *   Entity type object.
   * @param int $entity_id
   *   The entity id.
   *
   * @return string
   *   Returns entity uuid.
   */
  public function getEntityUuidById(ContentEntityType $entity_type, int $entity_id) {
    $query = $this->database->select($entity_type->get('base_table'), 't');
    $query->addField('t', 'uuid');
    $query->condition($entity_type->getKey('id'), $entity_id);
    $query->range(0, 1);
    return $query->execute()->fetchField();
  }

  /**
   * Get rendered content.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current entity.
   * @param string $view_mode
   *   The view mode.
   * @param string $langcode
   *   The language code.
   */
  public function getRenderedContent(EntityInterface $entity, $view_mode, $langcode) {
    $elements = $this->entityTypeManager
      ->getViewBuilder($entity->getEntityTypeId())
      ->view($entity, $view_mode, $langcode);
    return $this->renderer->renderPlain($elements);
  }

  /**
   * Export entity by view mode.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current entity.
   * @param string $view_mode
   *   The view mode.
   * @param string $langcode
   *   The language code.
   *
   * @return array
   *   Array of entities.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntityVariation(EntityInterface $entity, string $view_mode, string $langcode) {
    $rendered_data = $this->getRenderedContent($entity, $view_mode, $langcode);
    if ($entity->getEntityTypeId() == 'paragraph') {
      $url = NULL;
    }
    else {
      $urlObject = $entity->toUrl()->toString(TRUE);
      $url = $urlObject->getGeneratedUrl();
    }

    $preview_image = NULL;
    $config = $this->configFactory->get(EntityHelper::ENTITY_CONFIG_NAME);
    $config_image_preview_name = 'view_modes.' . $entity->getEntityTypeId() . '.' . $entity->bundle() . '.' . $view_mode . '.preview_image';
    $config_view_modes = $config->get($config_image_preview_name);
    if (!empty($config_view_modes)) {
      $preview_image_field = $entity->get($config_view_modes);
      if (!empty($preview_image_field) &&  !empty($preview_image_field->target_id)) {
        $preview_image = $this->fileUrlGenerator->generateAbsoluteString($preview_image_field->entity->uri->value);
      }
    }

    $result = [
      'content_uuid' => $entity->uuid(),
      'content_type' => $entity->getEntityTypeId(),
      'view_mode' => $view_mode,
      'language' => $langcode,
      'number_view' => 0,
      'label' => $entity->label(),
      'updated' => $this->dateFormatter->format($this->time->getCurrentTime(), 'custom', 'Y-m-d\TH:i:s'),
      'rendered_data' => $rendered_data,
      'base_url' => $this->getSiteDomainWithHost(),
      'url' => $url,
      'preview_image' => $preview_image,
    ];
    $taxonomy_relations = $this->getEntityTaxonomyRelations($entity);
    if ($taxonomy_relations) {
      $result['relations'] = $taxonomy_relations;
    }
    return $result;
  }

  /**
   * Get array of related taxonomy term fields/term uuids.
   *
   * Only taxonomies that are checked on Entity settings form.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The target entity.
   *
   * @return array
   *   Returns array of related taxonomy term fields/term uuids.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntityTaxonomyRelations(EntityInterface $entity) {
    $relations = [];
    $entity_type_id = $entity->getEntityTypeId();
    $bundle = $entity->bundle();
    $view_modes = $this->getEntityTypesConfig('view_modes');
    $available_taxonomies = [];
    if (isset($view_modes['taxonomy_term'])) {
      $available_taxonomies = array_keys($view_modes['taxonomy_term']);
    }
    $fields = $this->entityFieldManager
      ->getFieldDefinitions($entity_type_id, $bundle);
    foreach ($fields as $field) {
      if ($field instanceof FieldConfig
        && $field->getType() === 'entity_reference'
        && $field->getSetting('handler') === 'default:taxonomy_term'
      ) {
        $field_name = $field->getName();
        $settings = $field->getSetting('handler_settings');
        $field_taxonomies = $settings['target_bundles'];
        // Check if field contains at least one available taxonomy.
        if (count(array_intersect($available_taxonomies, $field_taxonomies)) == 0) {
          continue;
        }
        $terms = $entity->get($field_name)->getValue();
        $available_field_terms = [];
        foreach ($terms as $term) {
          $term_entity = $this->entityTypeManager
            ->getStorage('taxonomy_term')
            ->load($term['target_id']);
          $term_uuid = $term_entity->uuid();
          if (in_array($term_entity->bundle(), $available_taxonomies)) {
            $available_field_terms[] = $term_uuid;
          }
        }
        // Skip fields without any terms.
        if (!empty($available_field_terms)) {
          $relations[] = [
            'field' => $field_name,
            'terms' => $available_field_terms,
          ];
        }
      }
    }
    return $relations;
  }

  /**
   * Get list of available types > bundles > view modes.
   *
   * @return array|mixed|null
   *   Returns a list of available types > bundles > view modes.
   */
  public function getEntityTypesConfig() {
    return $this->configFactory
      ->get('acquia_perz.entity_config')
      ->get('view_modes');
  }

  /**
   * Get config item from acquia perz settings.
   *
   * @param string $config_item
   *   The name of the config item.
   *
   * @return string
   *   The value of the config item.
   */
  public function getSettingsConfigItem($config_item) {
    $settings = $this->configFactory->get('acquia_perz.settings');
    return $settings->get($config_item);
  }

  /**
   * Returns a unique hash for the current site.
   *
   * @return string
   *   A unique site hash, containing only alphanumeric characters.
   */
  public function getSiteHash() {
    if (!($hash = $this->state->get('acquia_perz.site_hash', FALSE))) {
      $base_url = $this->requestStack->getCurrentRequest()->getHost();
      $hash = substr(base_convert(hash('sha256', uniqid($base_url, TRUE)), 16, 36), 0, 6);
      $this->state->set('acquia_perz.site_hash', $hash);
    }
    return $hash;
  }

  /**
   * Returns a environment variable.
   *
   * @return string
   *   An acquia site environment.
   */
  public function getSiteEnvironment() {
    // During Beta, only production environment is supported.
    return 'prod';
  }

  /**
   * Get a site domain.
   *
   * @return string
   *   Returns a site domain.
   */
  public function getSiteDomain() {
    return $this->requestStack->getCurrentRequest()->getHost();
  }

  /**
   * Get a site domain with host.
   *
   * @return string
   *   Returns a site domain with host.
   */
  public function getSiteDomainWithHost() {
    return $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost();
  }

  /**
   * Run decision webhook.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function runDecisionWebhook(EntityInterface $entity = NULL) {
    $config = $this->configFactory->get('acquia_perz.settings');
    $decision_endpoint = $config->get('api.endpoint');
    if (empty($decision_endpoint)
      || $this->moduleHandler->moduleExists('acquia_perz_push')
      || !$this->isValidEntity($entity)) {
      return;
    }
    $data = [
      'account_id' => $this->getAccountId(),
      'origin' => $this->getSiteHash(),
      'environment' => $this->getSiteEnvironment(),
      'domain' => $this->getSiteDomain(),
      'entity_type' => $entity->getEntityTypeId(),
      'entity_uuid' => $entity->uuid(),
      'op' => 'create',
    ];
    $this->perzClient->pushEntity($data);
  }

  /**
   * Push variations to Personalization.
   *
   * @param string|null $account_id
   *   Account ID.
   * @param string|null $site_hash
   *   Site hash.
   * @param string|null $site_env
   *   Site environment.
   * @param array $entity_variations
   *   Entity variations.
   * @param string $op
   *   Operation.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function pushDataToPersonalization(?string $account_id, ?string $site_hash, ?string $site_env, array $entity_variations, string $op = 'normal') {
    if (!empty($account_id) && !empty($site_hash) && !empty($site_env)) {
      $data = [
        'account_id' => $account_id,
        'origin' => $site_hash,
        'environment' => $site_env,
        'entity_variations' => $entity_variations,
        'op' => $op,
      ];
      $this->perzClient->putVariations($data);
    }
  }

  /**
   * Check if acquia perz mock module is enabled.
   *
   * @return bool
   *   Returns TRUE if mock module is enabled.
   */
  public function isMockModuleExists() {
    return $this->moduleHandler->moduleExists('acquia_perz_mock');
  }

  /**
   * Get variations graphQL query.
   *
   * @return string
   *   Returns graphql query string.
   */
  public function getVariationsGraphqlQuery() {
    return "query getEntityVariations(\$entity_type_id: String!, \$entity_uuid: String!) {
        entity_variations(entity_type_id: \$entity_type_id, entity_uuid: \$entity_uuid) {
          content_uuid,
          label,
          content_type,
          view_mode,
          language,
          updated,
          rendered_data,
          relations {
            field,
            terms
          }
        }
      }";
  }

  /**
   * Validates a request signature.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return bool|KeyInterface
   *   TRUE if signature verification passes, FALSE otherwise.
   */
  public function validateSignature(Request $request) {
    return $this->perzClient->authenticate($request);
  }

  /**
   * Generate and add HMAC signature to reponse.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request.
   * @param \Symfony\Component\HttpFoundation\Response $response
   *   Response.
   * @param \Acquia\Hmac\KeyInterface $key
   *   Controller Key.
   *
   * @return \Psr\Http\Message\ResponseInterface
   *   Controller Key, FALSE otherwise.
   */
  public function signResponse(Request $request, Response $response, KeyInterface $key) {
    return $this->perzClient->generateResponseWithSignature($request, $response, $key);
  }

  /**
   * Collect imagefields for a particular entity bundle.
   *
   * @param string $entity_type_id
   *   The entity type identifier.
   * @param string $bundle
   *   The bundle name.
   *
   * @return array
   *   The array of image fields for a given entity bundle.
   */
  public function collectImageFields($entity_type_id, $bundle) {
    $image_fields = [];
    $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity_type_id, $bundle);
    foreach ($field_definitions as $field_key => $field_definition) {
      if ($field_definition->getType() === 'image') {
        $image_fields[$field_key] = $field_definition->getLabel();
      }
    }
    return $image_fields;
  }

  /**
   * Remove/update data from settings configuration.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param string $bundle
   *   The bundle of entity type.
   * @param string $view_mode
   *   The view_mode of bundle.
   * @param array $data
   *   User input data.
   * @param array $config_view_mode
   *   Config_view_mode of previous store data.
   */
  public function removeViewModeFromConfig($entity_type_id, $bundle, $view_mode, array $data, array &$config_view_mode) {
    $entity_view_perz_config = array_filter($data['view_mode']);
    if ((!empty($config_view_mode) && isset($config_view_mode[$entity_type_id][$bundle])
        && array_key_exists($view_mode, $config_view_mode[$entity_type_id][$bundle]))
      && empty($entity_view_perz_config)) {
      if (count($config_view_mode[$entity_type_id][$bundle]) > 1) {
        unset($config_view_mode[$entity_type_id][$bundle][$view_mode]);
      }
      elseif (count($config_view_mode[$entity_type_id]) > 1) {
        unset($config_view_mode[$entity_type_id][$bundle]);
      }
      else {
        unset($config_view_mode[$entity_type_id]);
      }
    }
    else {
      if (!empty($entity_view_perz_config)) {
        $role['render_role'] = $data['render_role'];
        $preview_image = isset($data['acquia_perz_preview_image']) ? ['preview_image' => $data['acquia_perz_preview_image']] : [];
        $config_view_mode[$entity_type_id][$bundle][$view_mode] = array_merge($role, $preview_image);
      }
    }
  }

  /**
   * Get account id.
   *
   * @return string
   *   Returns account id.
   */
  public function getAccountId() {
    return $this->subscription['account_id'];
  }

  /**
   * Search content in CIS.
   *
   * @param int $rows
   *   Number of rows to return in response.
   *
   * @return \Psr\Http\Message\ResponseInterface|void
   *   Response.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   *   Guzzle Exception.
   */
  public function searchContentInCis(int $rows = 10) {
    $data = [
      'account_id' => $this->getAccountId(),
      'origin' => $this->getSiteHash(),
      'environment' => $this->getSiteEnvironment(),
      'rows' => $rows,
    ];
    return $this->perzClient->getEntities($data);
  }

  /**
   * Delete content from CIS.
   *
   * @return \Psr\Http\Message\ResponseInterface|void
   *   Response.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   *   Guzzle Exception.
   */
  public function deleteContentFromCis() {
    $data = [
      'account_id' => $this->getAccountId(),
      'origin' => $this->getSiteHash(),
      'environment' => $this->getSiteEnvironment(),
    ];
    return $this->perzClient->deleteEntities($data);
  }

}
